/*
  ESP32_DS18x20
  board DOIT ESP32 DEVKIT V1
*/
#include <OneWire.h>

OneWire  ds(4);  // DATA von 1Wire-Bus an GPIO4 (mit 4.7K Widerstand nach Vdd 3,3V)
static int ROMx =0;

void setup(void) {
  Serial.begin(115200);
  //Versionspflege
  Serial.println("ESP32S_DS18x20_V4");
}

void loop(void) {
  byte i;
  byte present = 0;
  byte type_s;
  byte data[12];
  byte addr[8];
  float celsius, fahrenheit;

  if ( !ds.search(addr)) {        //Suchen nach dem nächsten Device. Das addr-Array ist  8 Byte lang
    Serial.println("Keine weitere device addresse.");
    Serial.println();
    ds.reset_search();  //Beginn einer neuen Suche. Der nächste Aufruf search(addr) beginnt wieder beim ersten Device
    delay(5250);
    ROMx =0;
    return;
  }

  ROMx ++;
  Serial.print(ROMx);
  Serial.print(". ROM");
  for( i = 0; i < 8; i++) {   // 8 Byte addr-Array einlesen
    Serial.write('-');
    Serial.print(addr[i], HEX);
  }

  Serial.print(" CRC=");
  Serial.print(addr[7], HEX);   //CRC von ROM
  
  if (OneWire::crc8(addr, 7) != addr[7]) {
      Serial.println("CRC is not valid!");
      return;
  }
 
  switch (addr[0]) {
    case 0x10:
      Serial.print("  Chip=DS18S20");  // or old DS1820
      type_s = 1;
      break;
    case 0x28:
      Serial.println("  Chip=DS18B20");
      type_s = 0;
      break;
    case 0x22:
      Serial.println("  Chip=DS1822");
      type_s = 0;
      break;
    default:
      Serial.println("Device is not a DS18x20 family device.");
      return;
  } 

  ds.reset();              // 1-Wire-Bus zurücksetzen. In der Regel wird dies benötigt, bevor mit irgendeinem Gerät kommuniziert wird 
  ds.select(addr);         // Auswahl eines Device anhand seiner Adresse
  ds.write(0x44, 0);       // Start Messung, parasitaere Versorgung aus
  delay(750);              // 750ms conversion time 
   
  present = ds.reset();   // 1-Wire-Bus zurücksetzen. In der Regel wird dies benötigt, bevor mit irgendeinem Gerät kommuniziert wird
  ds.select(addr);        // Auswahl eines Device anhand seiner Adresse
  ds.write(0xBE);         // Read Scratchpad

  Serial.print("   Data = ");
  Serial.print(present, HEX);
  Serial.print(" ");
  for ( i = 0; i < 9; i++) {        // 9 bytes Scratchpad einlesen
    data[i] = ds.read();
    Serial.print(data[i], HEX);
    Serial.print(" ");
  }
  Serial.print(" CRC=");
  Serial.print(OneWire::crc8(data, 8), HEX);  //CRC von Daten

  int16_t raw = (data[1] << 8) | data[0];
  if (type_s) {
    raw = raw << 3; // 9 bit resolution default
    if (data[7] == 0x10) {
      raw = (raw & 0xFFF0) + 12 - data[6];
    }
  } else {
    byte cfg = (data[4] & 0x60);
    if (cfg == 0x00) raw = raw & ~7;  // 9 bit resolution, 93.75 ms
    else if (cfg == 0x20) raw = raw & ~3; // 10 bit res, 187.5 ms
    else if (cfg == 0x40) raw = raw & ~1; // 11 bit res, 375 ms
  }
  celsius = (float)raw / 16.0;
  fahrenheit = celsius * 1.8 + 32.0;
  Serial.print("   Temperatur = ");
  Serial.print(celsius);
  Serial.print(" Celsius, ");
  Serial.print(fahrenheit);
  Serial.println(" Fahrenheit");
}
